# Stubs for Python 2.7 sha stdlib module

class sha(object):
    def update(self, arg: str) -> None: ...
    def digest(self) -> str: ...
    def hexdigest(self) -> str: ...
    def copy(self) -> sha: ...

def new(string: str = ...) -> sha: ...
blocksize: int
digest_size: int
