from distutils.cmd import Command
import sys

if sys.version_info >= (3,):
    class build_py(Command):
        def initialize_options(self) -> None: ...
        def finalize_options(self) -> None: ...
        def run(self) -> None: ...

    class build_py_2to3(build_py): ...
